//************************************************************************
// Example14Hanoi.java
//
// A solution of the classical Towers of Hanoi problem. We view each
// peg as a stack, and thus any single move of a disk is accomplished
// by means of two stack operations: 
// pop - take the disk from the top of the peg it is on.
// push - place the disk at the top of the peg it has to move to.
//************************************************************************

class Example14Hanoi
{
   public static void main (String[] args)
   {
      final int DISKS_NUMBER = Integer.parseInt(args[0]);
      Peg source      = new Peg (DISKS_NUMBER, "A"),
          auxiliary   = new Peg (DISKS_NUMBER, "B"),
          destination = new Peg (DISKS_NUMBER, "C");
      for (int i=0; i < DISKS_NUMBER; i++)
         source.push (DISKS_NUMBER - i);
      source.move (destination, auxiliary, DISKS_NUMBER);
   }// method main
}// class Example14Hanoi


class Peg
{
   private int[] disks;
   private int size;
   private String name;

   Peg (int maxSize, String name)
   {
      disks = new int[maxSize];
      size = 0;
      this.name = name;
   }// method Peg

   void push (int k)
   {
      disks[size] = k;
      size++;
   }// method push

   int pop()
   {
      size--;
      return disks[size];
   }// method pop

   void move (Peg destination, Peg auxiliary, int howMany)
   {
      if (howMany > 0)
      {
         move (auxiliary, destination, howMany - 1);
         int last = pop();
         destination.push (last);
         System.out.println ("Move disk " + last + " from " + this + " to "
                             + destination);
         auxiliary.move (destination, this, howMany - 1);
      }
   }// method move

   public String toString()
   {return name;}
}// class Peg
